CRAFTAX_CLASSIC_ACHIEVEMENTS = {
    0: "Collect Wood",
    1: "Place Table",
    2: "Eat Cow",
    3: "Collect Sapling",
    4: "Collect Drink",
    5: "Make Wood Pickaxe",
    6: "Make Wood Sword",
    7: "Place Plant",
    8: "Defeat Zombie",
    9: "Collect Stone",
    10: "Place Stone",
    11: "Eat Plant",
    12: "Defeat Skeleton",
    13: "Make Stone Pickaxe",
    14: "Make Stone Sword",
    15: "Wake Up",
    16: "Place Furnace",
    17: "Collect Coal",
    18: "Collect Iron",
    19: "Collect Diamond",
    20: "Make Iron Pickaxe",
    21: "Make Iron Sword",
}
CRAFTAX_CLASSIC_ACTION_MAPPING = {
    "noop": 0,
    "left": 1,
    "right": 2,
    "up": 3,
    "down": 4,
    "do": 5,
    "sleep": 6,
    "place_stone": 7,
    "place_table": 8,
    "place_furnace": 9,
    "place_plant": 10,
    "make_wood_pickaxe": 11,
    "make_stone_pickaxe": 12,
    "make_iron_pickaxe": 13,
    "make_wood_sword": 14,
    "make_stone_sword": 15,
    "make_iron_sword": 16,
}

CRAFTAX_CLASSIC_BACKDROP_BLOCK_TYPES = [
    "grass",
    "sand",
    "path",
]
CRAFTAX_CLASSIC_LOW_SALIENCE_OBJECTS = [
    "water",
    "stone",
    "tree",
    "wood",
    "plant",
]
CRAFTAX_CLASSIC_LOW_SALIENCE_MOBS = []
CRAFTAX_CLASSIC_HIGH_SALIENCE_OBJECTS = [
    "coal",
    "iron",
    "diamond",
    "crafting_table",
    "furnace",
    "lava",
    "ripe_plant",
]
CRAFTAX_CLASSIC_HIGH_SALIENCE_MOBS = ["Skeleton", "Zombie", "Cow", "Arrow"]
